#include "devices.h"
#include "wnvdsdk.h"

#define WM_GRAPHNOTIFY  WM_APP+1

// Declarations of global variables for use in grabbing.
IBaseFilter *g_pGrabberFilter = NULL;
ISampleGrabber *g_pGrabber = NULL;
unsigned int gWidth = 0;
unsigned int gHeight = 0;
unsigned int gChannels = 0;

HRESULT sgAddSampleGrabber(IGraphBuilder *pGraph)
{
        // Create the Sample Grabber.
        HRESULT hr = CoCreateInstance(CLSID_SampleGrabber, NULL, CLSCTX_INPROC_SERVER,
                                      IID_IBaseFilter, (void**) & g_pGrabberFilter);
        if (FAILED(hr)) {
                return hr;
        }
        hr = pGraph->AddFilter(g_pGrabberFilter, L"Sample Grabber");
        if (FAILED(hr)) {
                return hr;
        }

        g_pGrabberFilter->QueryInterface(IID_ISampleGrabber, (void**)&g_pGrabber);
        return hr;
}

HRESULT sgSetSampleGrabberMediaType()
{
        AM_MEDIA_TYPE mt;
        ZeroMemory(&mt, sizeof(AM_MEDIA_TYPE));
        mt.majortype = MEDIATYPE_Video;
        mt.subtype = MEDIASUBTYPE_RGB24;
        HRESULT hr = g_pGrabber->SetMediaType(&mt);
        if (FAILED(hr)) {
                return hr;
        }
        hr = g_pGrabber->SetOneShot(FALSE);
        hr = g_pGrabber->SetBufferSamples(TRUE);
        return hr;
}

void sgFreeMediaType(AM_MEDIA_TYPE& mt)
{
    if (mt.cbFormat != 0)
    {
        CoTaskMemFree((PVOID)mt.pbFormat);
        mt.cbFormat = 0;
        mt.pbFormat = NULL;
    }
    if (mt.pUnk != NULL)
    {
        // Unecessary because pUnk should not be used, but safest.
        mt.pUnk->Release();
        mt.pUnk = NULL;
    }
}

HRESULT sgGetSampleGrabberMediaType()
{
        AM_MEDIA_TYPE mt;
        HRESULT hr = g_pGrabber->GetConnectedMediaType(&mt);
        if (FAILED(hr)) {
                return hr;
        }

        VIDEOINFOHEADER *pVih = (VIDEOINFOHEADER *)mt.pbFormat;
        gChannels = pVih->bmiHeader.biBitCount / 8;
        gWidth = pVih->bmiHeader.biWidth;
        gHeight = pVih->bmiHeader.biHeight;

        sgFreeMediaType(mt);
        return hr;
}

// Declarations of global variables for use in video processing.
IBaseFilter *g_pSrc = NULL;
IMediaControl * g_pMC = NULL;
IMediaEventEx * g_pME = NULL;
IGraphBuilder * g_pGraph = NULL;
ICaptureGraphBuilder2 * g_pCapture = NULL;
IWnvDVidX    *g_dVidX = NULL;

IBaseFilter *g_pVMR = NULL;
IVMRFilterConfig9 *g_pFilterConfig = NULL;
IVMRMixerBitmap9 *g_pBMP = NULL;
VMR9AlphaBitmap	g_bmpInfo;
IVMRWindowlessControl9 *g_pWC = NULL;
IVideoWindow *g_pVW;

IVMRMixerControl9 *g_pMixer = NULL;

IBaseFilter *g_pSmartTee;

bool g_bStartup = true;

void ResizeVideoWindow(HWND hWnd)
{
    // Resize the video preview window to match owner window size
    //if (g_pVW)
    //{
    //    RECT rc;
    //    
    //    // Make the preview video fill our window
    //    GetClientRect(hWnd, &rc);
    //    g_pVW->SetWindowPosition(0, 0, rc.right, rc.bottom);
    //}
}

HRESULT InitializeGlobals(HWND hWnd)
{
	HRESULT hr;

	// Create the filter graph
	hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC,
		IID_IGraphBuilder, (void **) & g_pGraph);
	if (FAILED(hr))
		return hr;

	// Create the capture graph builder
	hr = CoCreateInstance(CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC, IID_ICaptureGraphBuilder2, (void **) & g_pCapture);
	if (FAILED(hr))
		return hr;

	// Obtain interfaces for media control and Video Window
	hr = g_pGraph->QueryInterface(IID_IMediaControl, (LPVOID *) & g_pMC);
	if (FAILED(hr))
		return hr;

	// Create video window?
	//hr = g_pGraph->QueryInterface(IID_IVideoWindow, (LPVOID *) &g_pVW);
	//g_pVW->put_Owner((OAHWND)hWnd);
	//hr = g_pVW->put_WindowStyle(WS_CHILD | WS_CLIPCHILDREN);
	//ResizeVideoWindow(hWnd);
	//hr = g_pVW->put_Visible(OATRUE);

	// Setup the media event
	hr = g_pGraph->QueryInterface(IID_IMediaEvent, (LPVOID *) & g_pME);
	if (FAILED(hr))
		return hr;

	// Setup the VMR9 interfaces
	hr = CoCreateInstance(CLSID_VideoMixingRenderer9, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&g_pVMR);
	
	// Filter configuration
	g_pVMR->QueryInterface(IID_IVMRFilterConfig9, (void **)&g_pFilterConfig);
	g_pFilterConfig->SetRenderingMode(VMR9Mode_Windowless);
	g_pFilterConfig->SetNumberOfStreams(1);

	// Setup windowless control
	hr = g_pVMR->QueryInterface(IID_IVMRWindowlessControl9, (void **)&g_pWC);

	RECT rect;
	GetClientRect(hWnd, &rect);
	rect.right = 640;
	rect.bottom = 480;
	hr = g_pWC->SetVideoClippingWindow(hWnd);
	g_pWC->SetVideoPosition(NULL, &rect);

	// Mixer setup
	g_pVMR->QueryInterface(IID_IVMRMixerControl9, (void **) &g_pMixer);
	DWORD dwPrefs=0;
	g_pMixer->GetMixingPrefs(&dwPrefs);
	dwPrefs |= MixerPref_PointFiltering;
	dwPrefs &= ~(MixerPref_BiLinearFiltering);
	g_pMixer->SetMixingPrefs(dwPrefs);

	// Bitmap overlay setup
	g_pVMR->QueryInterface(IID_IVMRMixerBitmap9, (void **)&g_pBMP);

	// Set the window handle used to process graph events
	hr = g_pME->SetNotifyWindow((OAHWND)hWnd, WM_GRAPHNOTIFY, 0);

	//
//	hr = CoCreateInstance(CLSID_SmartTee, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **) &g_pSmartTee);

	g_bStartup = false;
	return hr;
}

HRESULT GetCaptureDevice(IBaseFilter **ppSrc)
{
	HRESULT hr = S_OK;

	ICreateDevEnum *pDev = NULL;
	IEnumMoniker *pEMon = NULL;
	IMoniker *pMon = NULL;
	IBaseFilter *pSrc;

	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC, IID_ICreateDevEnum, (LPVOID *)&pDev);
	hr = pDev->CreateClassEnumerator(CLSID_VideoInputDeviceCategory, &pEMon, 0);
	ULONG numRet;
	hr = pEMon->Next(1, &pMon, &numRet);
	hr = pMon->BindToObject(0, 0, IID_IBaseFilter, (LPVOID *)&pSrc);

	//LPOLESTR str;
	//LPBC pbc;
	//IMoniker *prev = NULL;
	//hr = CreateBindCtx(0, &pbc);
	//for(int i = 0; i < 4; i++)
	//{
	//	prev = pMon;
	//	hr = pEMon->Next(1, &pMon, &numRet);
	//	pMon->GetDisplayName(pbc, prev, &str);
	//}

	if(SUCCEEDED(hr) && ppSrc != NULL)
	{
		*ppSrc = pSrc;
		(*ppSrc)->AddRef();
	}

	SAFE_RELEASE(pSrc);
	SAFE_RELEASE(pMon);
	SAFE_RELEASE(pEMon);
	SAFE_RELEASE(pDev);

	return hr;
}

HRESULT StartVideoStream(HWND hwnd)
{
	HRESULT hr = S_OK;

	InitializeGlobals(hwnd);

	// Add the filter graph to the capture graph.
	hr = g_pCapture->SetFiltergraph(g_pGraph);

	// Get the capture device.
	hr = GetCaptureDevice(&g_pSrc);
	hr = GetCaptureDevice(&g_pSrc);
	hr = GetCaptureDevice(&g_pSrc);
	hr = GetCaptureDevice(&g_pSrc);

	// Add the main video filter to the graph.
	hr = g_pGraph->AddFilter(g_pSrc, L"Video");

	// Add the video rendering filter to the graph.
	hr = g_pGraph->AddFilter(g_pVMR, L"Text");

	//
//	hr = g_pGraph->AddFilter(g_pSmartTee, L"SmartTee");

	// Add the grabber.
	//hr = sgAddSampleGrabber(g_pGraph);
	//hr = sgSetSampleGrabberMediaType();

	// Release the source filter.
//	SAFE_RELEASE(g_pSrc);

	hr = g_pCapture->RenderStream(
		&PIN_CATEGORY_PREVIEW,
		//&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video,
		g_pSrc,
		NULL,
		g_pVMR);
		//g_pVMR);

	// Start displaying the video.
	hr = g_pMC->Run();

	return hr;
}

HRESULT RenderText(HDC hdc)
{
	HRESULT hr = S_OK;
	if(g_bStartup)
		return hr;

	// Text output
	HDC compHDC = CreateCompatibleDC(hdc);
	HBITMAP bmp = CreateCompatibleBitmap(compHDC, 640, 480);
	SelectObject(compHDC, bmp);

    SetBkColor(compHDC, RGB(0, 0, 0));
//	SetBkMode(compHDC, TRANSPARENT);
	SetTextColor(compHDC, RGB(255, 0, 0));
	TextOut(compHDC, 100, 100, L"Testing stuff", 13);
	TextOut(compHDC, 100, 150, L"Testing stuff", 13);

	// Setup the bitmap alph info.
	RtlZeroMemory(&g_bmpInfo, sizeof(g_bmpInfo));
	g_bmpInfo.dwFlags |= VMRBITMAP_HDC | VMRBITMAP_SRCCOLORKEY;
	g_bmpInfo.hdc = compHDC;
	g_bmpInfo.rSrc.top = 0;
	g_bmpInfo.rSrc.bottom = 480;
	g_bmpInfo.rSrc.left = 0;
	g_bmpInfo.rSrc.right = 640;
	g_bmpInfo.rDest.top = 0.0f;
	g_bmpInfo.rDest.bottom = 1.0f;
	g_bmpInfo.rDest.left = 0.0f;
	g_bmpInfo.rDest.right = 1.0f;
	g_bmpInfo.fAlpha = 1.0f;
	g_bmpInfo.clrSrcKey = RGB(0, 0, 0);

	g_pBMP->SetAlphaBitmap(&g_bmpInfo);

	DeleteObject(bmp);
	DeleteObject(compHDC);

	return hr;
}
